Le design pattern  Observer en python
=====================================
![category](développement)
![tag](designpattern)
![tag](python)

```python
class Event(object):
    """ Classe événement """
    def __init__(self, name):
        self.name = name
        self._observateurs = []

    def __iadd__(self, observateur):
        """ ajoute un observateur """
        if not callable(observateur):
            raise TypeError("Doit être un objet appelable")
        observateurs = self._observateurs
        if observateur not in observateurs:
            observateurs.append(observateur)
        return self

    def __isub__(self, observateur):
        """ retire un observateur """
        observateurs = self._observateurs
        if observateur in observateurs:
            observers.remove(observateur)
        return self

    def __call__(self, *args, **kw):
        """ déclenche l'événement auprès de tous les inscrits """
        [observateur(*args, **kw) for observateur in self._observateurs]

```

utilisation

```python
class Fenetre(object):
    def __init__(self, name):
        self.name = name
        self.titre = 'titre de la fenêtre'
        self.contenu = 'contenu de la fenêtre'
        self.titreEvent = Event('titre')
        self.contenuEvent = Event('contenu')
        globalTitreEvent = Event('titre global')

    def changeTitre(self, titre):
        self.titre = titre
        self.titreEvent(titre)
        Fenetre.globalTitreEvent(self.name, titre)

    def changeContenu(self, contenu):
        self.titre = contenu
        self.contenuEvent(contenu)
        
class ObservateurTitre(object):
    def __init__(self, fenetre):
        fenetre.titreEvent += self
        
    def __call__(self, titre):
        print ('le titre a été changé en "%s" !' % titre)
        
class ObservateurContenu(object):
    def __init__(self, fenetre):
        fenetre.contenuEvent += self.contenuChange
    
    def contenuChange(self, contenu):
        print('le contenu a été changé en "%s" !' % contenu)
    
class ObservateurGlobal(object):
    def __init__(self, fenetre):
        fenetre.globalTitreEvent += self
    
    def __call__(self, name, titre):
        print('le titre de "%s" a été changé en "%s" !' % (name, titre))

fenetre = Fenetre('fenetre 1')
fenetre2 = Fenetre('fenetre 2')
ob1 = ObservateurTitre(fenetre)
ob2 = ObservateurContenu(fenetre)
ob3 = ObservateurGlobal(fenetre)
fenetre.changeTitre('nouveau titre')
fenetre.changeContenu('nouveau contenu')
fenetre2.changeTitre('nouveau titre2')```
